#include "stdafx.h"
#include "canvas.h"

#include "style.h"
#include "state.h"

namespace brixx
{

#define FONT_SIZE 10

static const Style	BUTTON(0xff3f3f3f, -1, 0x7fff0000),
					BUTTON_INACTIVE(0x3f6f6fff, 0x3f9f9fff, 0x1f0000ff),
					RADIO_ACTIVE(0xff7f7f7f, 0xffafafaf, 0xffff0000),
					RADIO_INACTIVE(0xff3f3f3f, 0xff7f7f7f, 0xff3f3f3f);

static void locRenderText(Gdiplus::Graphics& someGraphics, const unsigned int aColor, const int aX, const int aY, const int aWidth, const WCHAR* aText)
{
	const Gdiplus::FontFamily FAM(L"Verdana");
	const Gdiplus::Font F(&FAM, FONT_SIZE, Gdiplus::FontStyleRegular, Gdiplus::UnitPixel);
	const Gdiplus::Color C(aColor);
	const Gdiplus::SolidBrush B(C);
	Gdiplus::PointF p((float)aX, (float)aY);

	//centering
	if(aWidth > -1)
	{
		Gdiplus::RectF b;
		if(someGraphics.MeasureString(aText, ::wcslen(aText), &F, p, &b) == Gdiplus::Ok)
		{
			p.X += (aWidth - (b.GetRight() - b.GetLeft())) * 0.5f - 2.f;
		}
	}

	someGraphics.DrawString(aText, -1, &F, p, &B);
}

Canvas::Canvas(State& aState, const int aWidth, const int aHeight, const unsigned int aColor)
:myState(aState)
,myCanvas(aWidth, aHeight)
{
	assert(myCanvas.GetWidth() == aWidth);
	assert(myCanvas.GetHeight() == aHeight);
	myGraphics = Gdiplus::Graphics::FromImage(&myCanvas);

	Gdiplus::Color c(aColor);
	Gdiplus::SolidBrush b(c);
	myGraphics->FillRectangle(&b, 0, 0, aWidth, aHeight);
}

Canvas::~Canvas()
{
	delete myGraphics;
}

Gdiplus::Graphics& Canvas::getGraphics()
{
	return *myGraphics;
}

void Canvas::render(Gdiplus::Graphics& aTarget)
{
	aTarget.DrawImage(&myCanvas, 0, 0);
}

void Canvas::text(const int aX, const int aY, const WCHAR* aText, const unsigned int aColor, const int aWidth)
{
	if(aText)
	{
		if(aColor != 0xff000000)
			locRenderText(*myGraphics, 0xff3f3f3f, aX + 1, aX + 1, aWidth, aText);
		locRenderText(*myGraphics, aColor, aX, aY, aWidth, aText);
	}
}

const bool Canvas::button(const int aX, const int aY, const int aWidth, const WCHAR* aText, const bool anActive)
{
	bool ret;

	if(anActive)
	{
		ret = buttonRect(BUTTON, aX, aY, aWidth, myState.myButtonHeight) && wasPressed(VK_LBUTTON);
	}
	else
	{
		renderRect(false, false, BUTTON_INACTIVE, aX, aY, aWidth, myState.myButtonHeight);
		ret = false;
	}

	if(aText)
		text(aX, aY, aText, 0xff000000, aWidth);

	return ret;
}

const bool Canvas::radio(const bool anActive, const int aX, const int aY, const int aWidth, const WCHAR* aText)
{
	const bool RET(buttonRect(anActive ? RADIO_ACTIVE : RADIO_INACTIVE, aX, aY, aWidth, myState.myButtonHeight) && wasPressedInverted(VK_LBUTTON));

	if(aText)
		text(aX, aY, aText, 0xff000000, aWidth);

	return RET;
}


void Canvas::fillRect(const Gdiplus::Color& aColor, const int aX, const int aY, const int aWidth, const int aHeight) const
{
	myGraphics->FillRectangle(&Gdiplus::SolidBrush(aColor), aX, aY, aWidth, aHeight);
}

const int Canvas::buttonHeight() const
{
	return myState.myButtonHeight;
}

const bool Canvas::wasPressed(const int aKey) const
{
	return hasFocus() && myState.wasPressed(aKey);
}

const bool Canvas::hasFocus() const
{
	return myState.hasFocus();
}

const bool Canvas::cursorInsideGui() const
{
	return myState.myInsideRect;
}

//private
const bool Canvas::buttonRect(const Style& aStyle, const int aX, const int aY, const int aWidth, const int aHeight)
{
	//Rect& r(myRects[myNumRects]);
	//myNumRects++;

	////only set down style if not dragging
	//r.set(myState.isDown(VK_LBUTTON), aStyle, aX, aY, aWidth, aHeight);
	const bool INSIDE(insideRect(myState.cursorPos(), aX, aY, aWidth, aHeight));

	renderRect(INSIDE, myState.isDown(VK_LBUTTON), aStyle, aX, aY, aWidth, aHeight);

	//only set inside style if not dragging
	if(INSIDE)
	{
		myState.myInsideRect = true;
		return true;
	}

	return false;
}

void Canvas::renderRect(const bool anInside, const bool aDown, const Style& aStyle, const int aX, const int aY, const int aWidth, const int aHeight) const
{
	if(anInside)
	{
		if(aDown)
		{
			//myGraphics->FillRectangle(&Gdiplus::SolidBrush(aStyle.myOut), aX, aY, aWidth - 1, aHeight - 1);
			fillRect(aStyle.myOut, aX, aY, aWidth - 1, aHeight - 1);
		}
		else
		{
			//myGraphics->FillRectangle(&Gdiplus::SolidBrush(aStyle.myIn), aX, aY, aWidth - 1, aHeight - 1);
			fillRect(aStyle.myIn, aX, aY, aWidth - 1, aHeight - 1);
		}
	}
	else
	{
		myGraphics->FillRectangle(&Gdiplus::LinearGradientBrush(Gdiplus::PointF(0.f, (float)(aY)),
															Gdiplus::PointF(0.f, (float)(aY + aHeight - 1)),
															aStyle.myIn, aStyle.myOut),
															aX, aY, aWidth - 1, aHeight - 1);
	}

	myGraphics->DrawRectangle(&Gdiplus::Pen(aStyle.myEdge), aX, aY, aWidth - 1, aHeight - 1);
}

const bool Canvas::insideRect(const POINT& aPoint, const int aX, const int aY, const int aWidth, const int aHeight) const
{
	const int CPX(aPoint.x), CPY(aPoint.y);

	return CPX > aX && CPY > aY && CPX < (aX + aWidth) && CPY < (aY + aHeight);
}

const bool Canvas::isDown(const int aKey) const
{
	return hasFocus() && myState.isDown(aKey);
}

const bool Canvas::wasPressedInverted(const int aKey) const
{
	return hasFocus() && myState.wasPressedInverted(aKey);
}

const Vector2i Canvas::cursorPos() const
{
	const ::POINT P(myState.cursorPos());
	return Vector2i(P.x, P.y);
}

}